"""DO NOT MODIFY. Auto-generated by script/fingerprint_frontend."""

FINGERPRINTS = {
    "core.js": "769f3fdd4e04b34bd66c7415743cf7b5",
    "frontend.html": "d48d9a13f7d677e59b1d22c6db051207",
    "mdi.html": "7a0f14bbf3822449f9060b9c53bd7376",
    "micromarkdown-js.html": "93b5ec4016f0bba585521cf4d18dec1a",
    "panels/ha-panel-dev-event.html": "f19840b9a6a46f57cb064b384e1353f5",
    "panels/ha-panel-dev-info.html": "3765a371478cc66d677cf6dcc35267c6",
    "panels/ha-panel-dev-service.html": "1d223225c1c75083738033895ea3e4b5",
    "panels/ha-panel-dev-state.html": "8257d99a38358a150eafdb23fa6727e0",
    "panels/ha-panel-dev-template.html": "cbb251acabd5e7431058ed507b70522b",
    "panels/ha-panel-history.html": "9f2c72574fb6135beb1b381a4b8b7703",
    "panels/ha-panel-iframe.html": "d920f0aa3c903680f2f8795e2255daab",
    "panels/ha-panel-logbook.html": "313f2ac57aaa5ad55933c9bbf8d8a1e5",
    "panels/ha-panel-map.html": "13f120066c0b5faa2ce1db2c3f3cc486",
    "websocket_test.html": "575de64b431fe11c3785bf96d7813450"
}
